@@ -2,6 +2,7 @@ package main
 
 import (
 	"fmt"
+	"reflect"
 	"testing"
 
 	"github.com/skupperproject/skupper/api/types"
@@ -45,7 +46,7 @@ func TestGetServiceDefinitionFromAnnotatedDeployment(t *testing.T) {
 	}
 
 	// Help preparing sample deployments to compose test table
-	newDeployment := func(name string, proxyAnnotationProtocol string, containerPortAnnotation string, addressAnnotation string, containerPort int, labels map[string]string) *v1.Deployment {
+	newDeployment := func(name string, proxyAnnotationProtocol string, containerPortAnnotation string, addressAnnotation string, containerPort int, labels string, selectors map[string]string) *v1.Deployment {
 		// Add port to container if > 0
 		containerPorts := []corev1.ContainerPort{}
 		if containerPort > 0 {
@@ -72,12 +73,15 @@ func TestGetServiceDefinitionFromAnnotatedDeployment(t *testing.T) {
 		if addressAnnotation != "" {
 			annotations[types.AddressQualifier] = addressAnnotation
 		}
+		if labels != "" {
+			annotations[types.ServiceLabels] = labels
+		}
 
 		// Only initialize the selector pointer if a label has been provided
 		var selector *metav1.LabelSelector
-		if len(labels) > 0 {
+		if len(selectors) > 0 {
 			selector = &metav1.LabelSelector{
-				MatchLabels: labels,
+				MatchLabels: selectors,
 			}
 		}
 		return &v1.Deployment{
@@ -99,66 +103,69 @@ func TestGetServiceDefinitionFromAnnotatedDeployment(t *testing.T) {
 	}
 
 	// labels to use while preparing the test table
+	labels := "app=app1"
 	selectorWithLabels := map[string]string{"label1": "value1"}
 	selectorWithoutLabels := map[string]string{}
 
 	// test table below is meant to cover getServiceDefinitionFromAnnotatedDeployment()
 	testTable := []test{
-		{"no-proxy-annotation", newDeployment("dep1", "", "", "", 8080, selectorWithLabels), result{
+		{"no-proxy-annotation", newDeployment("dep1", "", "", "", 8080, labels, selectorWithLabels), result{
 			service: types.ServiceInterface{},
 			success: false,
 		}},
-		{"http-port-annotation-no-address", newDeployment("dep1", "http", "81", "", 8080, selectorWithLabels), result{
+		{"http-port-annotation-no-address", newDeployment("dep1", "http", "81", "", 8080, labels, selectorWithLabels), result{
 			service: types.ServiceInterface{
 				Address:  "dep1",
 				Protocol: "http",
-				Port:     81,
+				Ports:    []int{81},
 				Targets:  []types.ServiceInterfaceTarget{{Name: "dep1", Selector: "label1=value1"}},
+				Labels:   map[string]string{"app": "app1"},
 				Origin:   "annotation",
 			},
 			success: true,
 		}},
-		{"http-port-annotation-no-addess-without-selector", newDeployment("dep1", "http", "81", "", 8080, selectorWithoutLabels), result{
+		{"http-port-annotation-no-addess-without-selector", newDeployment("dep1", "http", "81", "", 8080, "", selectorWithoutLabels), result{
 			service: types.ServiceInterface{
 				Address:  "dep1",
 				Protocol: "http",
-				Port:     81,
+				Ports:    []int{81},
 				Targets:  []types.ServiceInterfaceTarget{{Name: "dep1", Selector: ""}},
 				Origin:   "annotation",
 			},
 			success: true,
 		}},
-		{"http-port-container-no-address", newDeployment("dep1", "http", "", "", 8080, selectorWithLabels), result{
+		{"http-port-container-no-address", newDeployment("dep1", "http", "", "", 8080, "", selectorWithLabels), result{
 			service: types.ServiceInterface{
 				Address:  "dep1",
 				Protocol: "http",
-				Port:     8080,
+				Ports:    []int{8080},
 				Targets:  []types.ServiceInterfaceTarget{{Name: "dep1", Selector: "label1=value1"}},
 				Origin:   "annotation",
 			},
 			success: true,
 		}},
-		{"http-no-port-no-address", newDeployment("dep1", "http", "", "", 0, selectorWithLabels), result{
+		{"http-no-port-no-address", newDeployment("dep1", "http", "", "", 0, "", selectorWithLabels), result{
 			service: types.ServiceInterface{
 				Address:  "dep1",
 				Protocol: "http",
-				Port:     80,
+				Ports:    []int{80},
 				Targets:  []types.ServiceInterfaceTarget{{Name: "dep1", Selector: "label1=value1"}},
 				Origin:   "annotation",
 			},
 			success: true,
 		}},
-		{"http-no-port-with-address", newDeployment("dep1", "http", "", "address1", 0, selectorWithLabels), result{
+		{"http-no-port-with-address", newDeployment("dep1", "http", "", "address1", 0, labels, selectorWithLabels), result{
 			service: types.ServiceInterface{
 				Address:  "address1",
 				Protocol: "http",
-				Port:     80,
+				Ports:    []int{80},
 				Targets:  []types.ServiceInterfaceTarget{{Name: "dep1", Selector: "label1=value1"}},
+				Labels:   map[string]string{"app": "app1"},
 				Origin:   "annotation",
 			},
 			success: true,
 		}},
-		{"tcp-invalid-port-no-address", newDeployment("dep1", "tcp", "invalid", "", 0, selectorWithLabels), result{
+		{"tcp-invalid-port-no-address", newDeployment("dep1", "tcp", "invalid", "", 0, "", selectorWithLabels), result{
 			service: types.ServiceInterface{},
 			success: false,
 		}},
@@ -169,7 +176,7 @@ func TestGetServiceDefinitionFromAnnotatedDeployment(t *testing.T) {
 		t.Run(test.name, func(t *testing.T) {
 			service, success := dm.getServiceDefinitionFromAnnotatedDeployment(test.deployment)
 			// Validating returned service
-			assert.Equal(t, test.expected.service.Port, service.Port)
+			assert.Assert(t, reflect.DeepEqual(test.expected.service.Ports, service.Ports))
 			assert.Equal(t, test.expected.service.Protocol, service.Protocol)
 			assert.Equal(t, test.expected.service.Address, service.Address)
 			assert.Equal(t, len(test.expected.service.Targets), len(service.Targets))
@@ -177,6 +184,7 @@ func TestGetServiceDefinitionFromAnnotatedDeployment(t *testing.T) {
 				assert.Equal(t, test.expected.service.Targets[0].Name, service.Targets[0].Name)
 				assert.Equal(t, test.expected.service.Targets[0].Selector, service.Targets[0].Selector)
 			}
+			assert.DeepEqual(t, test.expected.service.Labels, service.Labels)
 			assert.Equal(t, test.expected.service.Origin, service.Origin)
 			// Validating overall result
 			assert.Equal(t, success, test.expected.success)
@@ -212,7 +220,7 @@ func TestGetServiceDefinitionFromAnnotatedService(t *testing.T) {
 	}
 
 	// Helper used to prepare test table
-	annotatedService := func(name string, proxyAnnotationProtocol string, addressAnnotation string, targetAnnotation string, labels map[string]string, originalSelector string, originalTargetPort string, targetPort int, ports ...int) *corev1.Service {
+	annotatedService := func(name string, proxyAnnotationProtocol string, addressAnnotation string, targetAnnotation string, labels string, selectorMap map[string]string, originalSelector string, originalTargetPort string, targetPorts []int, ports ...int) *corev1.Service {
 
 		annotations := map[string]string{}
 		if proxyAnnotationProtocol != "" {
@@ -233,10 +241,14 @@ func TestGetServiceDefinitionFromAnnotatedService(t *testing.T) {
 			annotations[types.OriginalTargetPortQualifier] = originalTargetPort
 		}
 
+		if labels != "" {
+			annotations[types.ServiceLabels] = labels
+		}
+
 		// Only initialize the selector pointer if a label has been provided
 		var selectors map[string]string
-		if len(labels) > 0 {
-			selectors = labels
+		if len(selectorMap) > 0 {
+			selectors = selectorMap
 		}
 
 		// Only set ports, if at least one provided
@@ -246,7 +258,7 @@ func TestGetServiceDefinitionFromAnnotatedService(t *testing.T) {
 				svcPorts = append(svcPorts, corev1.ServicePort{
 					Name:       fmt.Sprintf("port%d", i),
 					Port:       int32(port),
-					TargetPort: intstr.FromInt(targetPort),
+					TargetPort: intstr.FromInt(targetPorts[i]),
 				})
 			}
 		}
@@ -267,10 +279,10 @@ func TestGetServiceDefinitionFromAnnotatedService(t *testing.T) {
 	// Create fake target services
 	var err error
 	// good path with target service providing port
-	_, err = vanClient.KubeClient.CoreV1().Services(NS).Create(annotatedService("targetsvc", "", "", "", nil, "", "", 0, 8888))
+	_, err = vanClient.KubeClient.CoreV1().Services(NS).Create(annotatedService("targetsvc", "", "", "", "app=app1", nil, "", "", []int{0}, 8888))
 	assert.NilError(t, err)
 	// this is used to test case when protocol is http but target service does not provide a port, so it uses 80
-	_, err = vanClient.KubeClient.CoreV1().Services(NS).Create(annotatedService("targetsvcnoport", "", "", "", nil, "", "", 0))
+	_, err = vanClient.KubeClient.CoreV1().Services(NS).Create(annotatedService("targetsvcnoport", "", "", "", "app=app2", nil, "", "", []int{0}))
 	assert.NilError(t, err)
 
 	// Mock error when trying to get info for badtargetsvc
@@ -283,105 +295,109 @@ func TestGetServiceDefinitionFromAnnotatedService(t *testing.T) {
 	})
 
 	testTable := []test{
-		{"no-proxy", annotatedService("", "", "", "", nil, "", "", 0), result{
+		{"no-proxy", annotatedService("", "", "", "", "", nil, "", "", []int{0}), result{
 			service: types.ServiceInterface{},
 			success: false,
 		}},
-		{"no-target-no-selector", annotatedService("svc", "http", "", "", nil, "", "", 0), result{
+		{"no-target-no-selector", annotatedService("svc", "http", "", "", "", nil, "", "", []int{0}), result{
 			service: types.ServiceInterface{
 				Address:  "svc",
 				Protocol: "http",
-				Port:     0,
+				Ports:    []int{},
 			},
 			success: false,
 		}},
-		{"http-8080-targetsvc-8888", annotatedService("svc", "http", "address", "targetsvc", nil, "", "", 0, 8080), result{
+		{"http-8080-targetsvc-8888", annotatedService("svc", "http", "address", "targetsvc", "app=app1", nil, "", "", []int{8888}, 8080), result{
 			service: types.ServiceInterface{
 				Address:  "address",
 				Protocol: "http",
-				Port:     8080,
+				Ports:    []int{8080},
 				Targets: []types.ServiceInterfaceTarget{
 					{
-						Name:       "targetsvc",
-						Selector:   "",
-						TargetPort: 8888,
-						Service:    "targetsvc",
+						Name:        "targetsvc",
+						Selector:    "",
+						TargetPorts: map[int]int{8080: 8888},
+						Service:     "targetsvc",
 					},
 				},
+				Labels: map[string]string{
+					"app": "app1",
+				},
 				Origin: "annotation",
 			},
 			success: true,
 		}},
-		{"http-80-targetsvcnoport", annotatedService("svc", "http", "address", "targetsvcnoport", nil, "", "", 0), result{
+		{"http-80-targetsvcnoport", annotatedService("svc", "http", "address", "targetsvcnoport", "app=app1", nil, "", "", []int{}), result{
 			service: types.ServiceInterface{
 				Address:  "address",
 				Protocol: "http",
-				Port:     80,
+				Ports:    []int{80},
 				Targets: []types.ServiceInterfaceTarget{
 					{
-						Name:       "targetsvcnoport",
-						Selector:   "",
-						TargetPort: 0,
-						Service:    "targetsvcnoport",
+						Name:     "targetsvcnoport",
+						Selector: "",
+						Service:  "targetsvcnoport",
 					},
 				},
+				Labels: map[string]string{
+					"app": "app1",
+				},
 				Origin: "annotation",
 			},
 			success: true,
 		}},
-		{"tcp-noport-targetsvcnoport", annotatedService("svc", "tcp", "address", "targetsvcnoport", nil, "", "", 0), result{
+		{"tcp-noport-targetsvcnoport", annotatedService("svc", "tcp", "address", "targetsvcnoport", "", nil, "", "", []int{0}), result{
 			service: types.ServiceInterface{
 				Address:  "address",
 				Protocol: "tcp",
-				Port:     0,
+				Ports:    []int{},
 			},
 			success: false,
 		}},
-		{"tcp-noport-targetsvc-8888", annotatedService("svc", "tcp", "address", "targetsvc", nil, "", "", 0), result{
+		{"tcp-noport-targetsvc-8888", annotatedService("svc", "tcp", "address", "targetsvc", "", nil, "", "", []int{}), result{
 			service: types.ServiceInterface{
 				Address:  "address",
 				Protocol: "tcp",
-				Port:     8888,
+				Ports:    []int{8888},
 				Targets: []types.ServiceInterfaceTarget{
 					{
-						Name:       "targetsvc",
-						Selector:   "",
-						TargetPort: 0,
-						Service:    "targetsvc",
+						Name:        "targetsvc",
+						Selector:    "",
+						TargetPorts: map[int]int{8888: 8888},
+						Service:     "targetsvc",
 					},
 				},
 				Origin: "annotation",
 			},
 			success: true,
 		}},
-		{"bad-target-service", annotatedService("svc", "http", "address", "badtargetsvc", nil, "", "", 0, 8080), result{
+		{"bad-target-service", annotatedService("svc", "http", "address", "badtargetsvc", "", nil, "", "", []int{0}, 8080), result{
 			service: types.ServiceInterface{
 				Address:  "address",
 				Protocol: "http",
-				Port:     8080,
+				Ports:    []int{8080},
 				Targets: []types.ServiceInterfaceTarget{
 					{
-						Name:       "badtargetsvc",
-						Selector:   "",
-						TargetPort: 0,
-						Service:    "badtargetsvc",
+						Name:     "badtargetsvc",
+						Selector: "",
+						Service:  "badtargetsvc",
 					},
 				},
 				Origin: "annotation",
 			},
 			success: true,
 		}},
-		{"tcp-noport-targetsvc-8888", annotatedService("svc", "tcp", "address", "targetsvc", nil, "", "", 0), result{
+		{"tcp-noport-targetsvc-8888", annotatedService("svc", "tcp", "address", "targetsvc", "", nil, "", "", []int{}), result{
 			service: types.ServiceInterface{
 				Address:  "address",
 				Protocol: "tcp",
-				Port:     8888,
+				Ports:    []int{8888},
 				Targets: []types.ServiceInterfaceTarget{
 					{
-						Name:       "targetsvc",
-						Selector:   "",
-						TargetPort: 0,
-						Service:    "targetsvc",
+						Name:        "targetsvc",
+						Selector:    "",
+						TargetPorts: map[int]int{8888: 8888},
+						Service:     "targetsvc",
 					},
 				},
 				Origin: "annotation",
@@ -389,20 +405,20 @@ func TestGetServiceDefinitionFromAnnotatedService(t *testing.T) {
 			success: true,
 		}},
 		{"tcp-noport-selector", annotatedService("svc", "tcp", "address", "",
-			map[string]string{"label1": "value1"}, "", "", 0), result{
+			"", map[string]string{"label1": "value1"}, "", "", []int{0}), result{
 			service: types.ServiceInterface{
 				Address:  "address",
 				Protocol: "tcp",
-				Port:     0,
+				Ports:    []int{},
 			},
 			success: false,
 		}},
 		{"http-noport-selector", annotatedService("svc", "http", "address", "",
-			map[string]string{"label1": "value1"}, "", "", 0), result{
+			"", map[string]string{"label1": "value1"}, "", "", []int{0}), result{
 			service: types.ServiceInterface{
 				Address:  "address",
 				Protocol: "http",
-				Port:     80,
+				Ports:    []int{80},
 				Targets: []types.ServiceInterfaceTarget{
 					{
 						Name:     "svc",
@@ -414,16 +430,16 @@ func TestGetServiceDefinitionFromAnnotatedService(t *testing.T) {
 			success: true,
 		}},
 		{"http-8080-selector", annotatedService("svc", "http", "address", "",
-			map[string]string{"label1": "value1"}, "", "", 8888, 8080), result{
+			"", map[string]string{"label1": "value1"}, "", "", []int{8888}, 8080), result{
 			service: types.ServiceInterface{
 				Address:  "address",
 				Protocol: "http",
-				Port:     8080,
+				Ports:    []int{8080},
 				Targets: []types.ServiceInterfaceTarget{
 					{
-						Name:       "svc",
-						Selector:   "label1=value1",
-						TargetPort: 8888,
+						Name:        "svc",
+						Selector:    "label1=value1",
+						TargetPorts: map[int]int{8080: 8888},
 					},
 				},
 				Origin: "annotation",
@@ -432,18 +448,18 @@ func TestGetServiceDefinitionFromAnnotatedService(t *testing.T) {
 		}},
 		{"http-8080-original-selector",
 			annotatedService("svc", "http", "address", "",
-				map[string]string{types.ComponentAnnotation: types.RouterComponent, types.ProxyQualifier: "http"},
-				"label1=value1", "8080", 1024, 8080),
+				"", map[string]string{types.ComponentAnnotation: types.RouterComponent, types.ProxyQualifier: "http"},
+				"label1=value1", "8080", []int{1024}, 8080),
 			result{
 				service: types.ServiceInterface{
 					Address:  "address",
 					Protocol: "http",
-					Port:     8080,
+					Ports:    []int{8080},
 					Targets: []types.ServiceInterfaceTarget{
 						{
-							Name:       "svc",
-							Selector:   "label1=value1",
-							TargetPort: 8080,
+							Name:        "svc",
+							Selector:    "label1=value1",
+							TargetPorts: map[int]int{8080: 8080},
 						},
 					},
 					Origin: "annotation",
@@ -451,13 +467,13 @@ func TestGetServiceDefinitionFromAnnotatedService(t *testing.T) {
 				success: true,
 			},
 		},
-		{"http-8080-bad-original-selector", annotatedService("svc", "http", "", "", map[string]string{
+		{"http-8080-bad-original-selector", annotatedService("svc", "http", "", "", "", map[string]string{
 			types.ComponentAnnotation: types.RouterComponent,
-		}, "", "", 0, 8080), result{
+		}, "", "", []int{0}, 8080), result{
 			service: types.ServiceInterface{
 				Address:  "svc",
 				Protocol: "http",
-				Port:     8080,
+				Ports:    []int{8080},
 				Targets:  []types.ServiceInterfaceTarget{},
 			},
 			success: false,
@@ -468,15 +484,16 @@ func TestGetServiceDefinitionFromAnnotatedService(t *testing.T) {
 		t.Run(test.name, func(t *testing.T) {
 			service, success := dm.getServiceDefinitionFromAnnotatedService(test.service)
 			assert.Equal(t, test.expected.success, success)
-			assert.Equal(t, test.expected.service.Port, service.Port)
+			assert.Assert(t, reflect.DeepEqual(test.expected.service.Ports, service.Ports))
 			assert.Equal(t, test.expected.service.Address, service.Address)
 			assert.Equal(t, len(test.expected.service.Targets), len(service.Targets))
 			if len(service.Targets) > 0 {
 				assert.Equal(t, test.expected.service.Targets[0].Name, service.Targets[0].Name)
 				assert.Equal(t, test.expected.service.Targets[0].Service, service.Targets[0].Service)
-				assert.Equal(t, test.expected.service.Targets[0].TargetPort, service.Targets[0].TargetPort)
+				assert.Assert(t, reflect.DeepEqual(test.expected.service.Targets[0].TargetPorts, service.Targets[0].TargetPorts))
 				assert.Equal(t, test.expected.service.Targets[0].Selector, service.Targets[0].Selector)
 			}
+			assert.DeepEqual(t, test.expected.service.Labels, service.Labels)
 			assert.Equal(t, test.expected.service.Origin, service.Origin)
 		})
 	}
@@ -487,17 +504,22 @@ func TestDeducePort(t *testing.T) {
 
 	// Helps generating the test table
 	type test struct {
-		name         string
-		deployment   *v1.Deployment
-		expectedPort int
+		name          string
+		deployment    *v1.Deployment
+		expectedPorts map[int]int
 	}
 
-	newDeployment := func(portAnnotation string, portContainer int) *v1.Deployment {
+	newDeployment := func(portAnnotation string, portContainer ...int) *v1.Deployment {
 		annotationMap := map[string]string{}
 		if portAnnotation != "" {
 			annotationMap["skupper.io/port"] = portAnnotation
 		}
 
+		ports := []corev1.ContainerPort{}
+		for _, p := range portContainer {
+			ports = append(ports, corev1.ContainerPort{ContainerPort: int32(p)})
+		}
+
 		return &v1.Deployment{
 			ObjectMeta: metav1.ObjectMeta{
 				Name:        "dep1",
@@ -508,9 +530,7 @@ func TestDeducePort(t *testing.T) {
 				Template: corev1.PodTemplateSpec{
 					Spec: corev1.PodSpec{
 						Containers: []corev1.Container{{
-							Ports: []corev1.ContainerPort{{
-								ContainerPort: int32(portContainer),
-							}},
+							Ports: ports,
 						}},
 					},
 				},
@@ -519,14 +539,16 @@ func TestDeducePort(t *testing.T) {
 	}
 
 	testTable := []test{
-		{"no-annotation-container-port", newDeployment("", 8080), 8080},
-		{"valid-annotation-container-port", newDeployment("8888", 8080), 8888},
-		{"invalid-annotation-container-port", newDeployment("invalid", 8080), 0},
+		{"no-annotation-container-port", newDeployment("", 8080), map[int]int{8080: 8080}},
+		{"no-annotation-container-ports", newDeployment("", 8080, 9090), map[int]int{8080: 8080, 9090: 9090}},
+		{"valid-annotation-container-port", newDeployment("8888", 8080), map[int]int{8888: 8888}},
+		{"valid-annotation-container-diff-ports", newDeployment("8888:8080", 8080), map[int]int{8888: 8080}},
+		{"invalid-annotation-container-port", newDeployment("invalid", 8080), map[int]int{}},
 	}
 
 	for _, test := range testTable {
 		t.Run(test.name, func(t *testing.T) {
-			assert.Assert(t, test.expectedPort == deducePort(test.deployment))
+			assert.Assert(t, reflect.DeepEqual(test.expectedPorts, deducePort(test.deployment)))
 		})
 	}
 }
@@ -535,7 +557,7 @@ func TestDeducePortFromService(t *testing.T) {
 	type test struct {
 		name     string
 		service  *corev1.Service
-		expected int
+		expected map[int]int
 	}
 
 	// Helper used to prepare test table
@@ -558,34 +580,36 @@ func TestDeducePortFromService(t *testing.T) {
 	}
 
 	testTable := []test{
-		{"no-port", newService(), 0},
-		{"one-port", newService(8080), 8080},
-		{"two-ports", newService(8080, 8081), 8080},
+		{"no-port", newService(), map[int]int{}},
+		{"one-port", newService(8080), map[int]int{8080: 8080}},
+		{"two-ports", newService(8080, 8081), map[int]int{8080: 8080, 8081: 8081}},
 	}
 
 	for _, test := range testTable {
 		t.Run(test.name, func(t *testing.T) {
-			assert.Equal(t, deducePortFromService(test.service), test.expected)
+			deducedPorts := deducePortFromService(test.service)
+			assert.Assert(t, reflect.DeepEqual(deducedPorts, test.expected), "got: %v - expected: %v", deducedPorts, test.expected)
 		})
 	}
 }
 
-func TestDeduceTargetPortFromService(t *testing.T) {
+func TestDeducePortFromServiceWithTargets(t *testing.T) {
 	type test struct {
 		name     string
 		service  *corev1.Service
-		expected int
+		expected map[int]int
 	}
 
 	// Helper used to prepare test table
-	newService := func(ports ...int) *corev1.Service {
+	newService := func(ports ...[]int) *corev1.Service {
 		// Only set ports, if at least one provided
 		var svcPorts []corev1.ServicePort
 		if len(ports) > 0 {
 			for i, port := range ports {
 				svcPorts = append(svcPorts, corev1.ServicePort{
 					Name:       fmt.Sprintf("port%d", i),
-					TargetPort: intstr.FromInt(port),
+					Port:       int32(port[0]),
+					TargetPort: intstr.FromInt(port[1]),
 				})
 			}
 		}
@@ -597,14 +621,15 @@ func TestDeduceTargetPortFromService(t *testing.T) {
 	}
 
 	testTable := []test{
-		{"no-port", newService(), 0},
-		{"one-port", newService(8080), 8080},
-		{"two-ports", newService(8080, 8081), 8080},
+		{"no-port", newService(), map[int]int{}},
+		{"one-port", newService([]int{80, 8080}), map[int]int{80: 8080}},
+		{"two-ports", newService([]int{80, 8080}, []int{81, 8081}), map[int]int{80: 8080, 81: 8081}},
 	}
 
 	for _, test := range testTable {
 		t.Run(test.name, func(t *testing.T) {
-			assert.Equal(t, deduceTargetPortFromService(test.service), test.expected)
+			deducedPorts := deducePortFromService(test.service)
+			assert.Assert(t, reflect.DeepEqual(deducedPorts, test.expected))
 		})
 	}
 }
